with Posicions; use Posicions;
with Ada.Real_Time; use Ada.Real_Time;
with Llista;
with Taulers; use Taulers;
with TableroVisual; use TableroVisual;
with Sorpreses; use Sorpreses;
with Cleaner; use Cleaner;

package ComecocosPack is
  
  type PDuration is access Duration;
  
  -- Predefinicion del Coordinador --
  type Coordinador;
  type PCoordinador is access Coordinador;
  
  -- Tasca comecocos --
  -- Cada cierto tiempo pedir una nueva posicin al coordinador para moverse.
  task type Comecocos(C: PCoordinador; P: PPosicio; Duracion: PDuration; incubado: boolean) is
    entry start;
    entry theEnd;
  end Comecocos;
  type PComecocos is access Comecocos;

  -- Lista de punteros a comecocos --
  package LComecocos is new Llista(PComecocos);
  use LComecocos;
  type PLlista is access LComecocos.Llista;
  
  -- Predinicion del Control
  type Control;
  type PControl is access Control;
  
  -- Coordinador --
  -- Indicar a los comecocos donde deben moverse y enviar mensajes a Control
  -- para crear nuevos comecocos incubados.
  -- Obligatorio usarse el SetControl para asignarle un controlador antes de ser usado
  -- Es una forma de pasarle la direccin del Control que sera creado despues de el.
  task type Coordinador(T: PTauler) is
    entry start;
    entry theEnd;
    entry SetControl(Ctrl: PControl);

    -- Al llamar noMove dejara dar posiciones nuevas para mover comecocos, sirve para parar
    -- el sistema. Ser llamado por una taska PartidaTimer, nunca por Control o
    -- se produciria un DeadLock.
    -- Al no mover, tampoco crear nuevos comecocos y asi evitamos un DeadLock
    -- entre Control - Comecocos - Coordinador.
    entry noMove;

    -- Los parametros C y Duracion son necesarios para los casos en los que se ha incubado
    -- y debe crearse un nuevo comecocos. Periodo es necesario para poder actualizar
    -- el tiempo de movimiento en la ventana de Status.
    entry donde_mov(veryOldP, oldP: in Posicio; P: out Posicio; Duracion: in PDuration; Periodo: in Time_Span);
  end Coordinador;
  
  -- Control --
  -- Iniciar y parar las tasca Coordinador y todos los comecocos. Ademas
  -- crea comecocos nuevos por orden del coordinador. Antes de enviarle un theEnd
  -- otro proceso exterior (PartidaTimer) debe haberle enviado al Coordinador un
  -- noIncubar para evitar DeadLocks.
  task type Control(T: PTauler; Coord: PCoordinador; Llistat: PLlista; Sorpreses: PSor; GarbageCleaner: PClean) is
    entry start;
    entry theEnd;
    entry create(Pos: in Posicio; Duracion: in PDuration; Periodo: in Time_Span);
  end Control;
  

end ComecocosPack;
