-----------------------------
-- Programaci Concurrent  --
--                         --
-- 20-Nov-2002             --
--                         --
-- Sergio Gomez            --
-- Alex Arenas             --
-----------------------------
with Text_IO; use Text_IO;
with Unchecked_Deallocation;

package body llista is

  --------------------
  -- Allibera_Espai --
  --------------------
  -- Allibera memoria dinamica  
  procedure Allibera_Espai is new Unchecked_Deallocation (Object => Node, Name => PNode);

  -- Inicializa una lista creando el ghost element
  procedure Crear(L: in out Llista) is
    Ghost: PNode;
  begin
    Ghost := new Node;
    L.Primer := Ghost;
    L.Primer.ALL.Next := null;
    L.Anterior := Ghost;

    EXCEPTION
      WHEN Storage_Error => Put_Line("No queda ms espai a memoria!");
  end Crear;
  
  -- En destination s'ha de passar una variable tipus Llista sense
  -- inicialitzar. Duplica una llista.
  procedure Copiar(Source: in Llista; Destination: in out Llista) is
    LAux: Llista := Source;
  begin
    Primer(LAux);
    Crear(Destination);
    while Queden(LAux) loop
      Afegir(Destination, Actual(LAux));
      Seguent(LAux);
    end loop;
  end Copiar;
  
  --------------------
  -- Afegir --
  --------------------
  -- Aade un elemento donde se encuentre el puntero Anterior.Next
  procedure Afegir (L: in out Llista; P: in Elem) is
    Nou: PNode;
    PDI: PNode;
  begin
    Nou := new Node;
    Nou.ALL.P := P;
    PDI := L.Anterior.ALL.Next;
    L.Anterior.ALL.Next := Nou;
    Nou.ALL.Next := PDI;
    L.Anterior := Nou;
    L.Total := L.Total+1;
    
    EXCEPTION
      WHEN Storage_Error => Put_Line("No queda ms espai a memoria!");
  end Afegir;

  -- Borra un elemento liberando su espacio
  procedure Esborrar(L: in out Llista) is
    PDI: PNode;
  begin
    PDI := L.Anterior.ALL.Next;
    if PDI /= null then
      L.Anterior.ALL.Next := PDI.ALL.Next;
      Allibera_Espai(PDI);
      L.Total := L.Total-1;
    end if;
  end Esborrar;
  ----------------------
  -- Existeix --
  ----------------------
  -- Busca la existencia de un elemento.
  function Existeix(L: in Llista; P: in Elem) return Boolean is
    Pos: Elem;
    LAux: Llista := L;
  begin
    Primer(LAux);
    while Queden(LAux) loop
      Pos := Actual(LAux);
      if Pos = P then
        return True;
      end if;
      Seguent(LAux);
    end loop;
    
    if Pos = P then
      return True;
    else
      return False;
    end if;
  end Existeix;

  
  -- Busca un elemento y lo modifica por otro
  -- Puede lanzar una excepcion si no existe la peticion que ha de modificar
  procedure Modificar(L: in out Llista; AntigaP, P: in Elem) is
    Pos: Elem;
    NoExisteix: Exception;
  begin
    Primer(L);
    while Queden(L) loop
      Pos := Actual(L);
      if Pos = AntigaP then
        exit;
      end if;
      Seguent(L);
    end loop;
    
    if Pos = AntigaP then
      L.Anterior.Next.ALL.P := P;
    else
      raise NoExisteix;
    end if;
    
    --EXCEPTION
      --WHEN NoExisteix => ??
  end Modificar;
  
  function Total(L: Llista) return integer is
  begin
    return L.Total;
  end Total;
  
  
  ---------------------
  -- Primer --
  ---------------------
  -- Situa el puntero Anterior en el ghost element
  procedure Primer (L: in out Llista) is
  begin
    L.Anterior := L.Primer;
  end Primer;

  -- Retorna el elemento apuntado por Anterior.Next
  function Actual(L: in Llista) return Elem is
  begin
    return L.Anterior.ALL.Next.ALL.P;
  end Actual;
  
  
  ----------------------
  -- Queden_Posicions --
  ----------------------
  -- Comprueba si despues del Objeto apuntado por Anterior.Next hay otro
  function Queden (L: in Llista) return Boolean is
    PDI: PNode;
  begin
    PDI := L.Anterior.ALL.Next;
    if PDI=Null then
      return False;
    else
      return True;
    end if;
  end Queden;

  
  -- Comprueba si la lista ya ha sido inicializada con Crear
  function Esta_Creada(L: in Llista) return Boolean is
  begin
    return not (L.Primer = null);
  end Esta_Creada;
  
  ---------------------
  -- Seguent --
  ---------------------
  -- Avanza al siguiente elemento el puntero Anterior
  procedure Seguent (L: in out Llista) is
  begin
    if Queden (L) then
      L.Anterior := L.Anterior.ALL.Next;
    end if;
  end Seguent;



  -----------------------
  -- Alliberar_Memoria --
  -----------------------
  -- Llibera l'espai de memoria on es troba una llista
  procedure Alliberar_Memoria (L: in out Llista) is
    Antaux:PNode;
  begin
    Primer(L);
    while Queden(L) loop
      Antaux:=L.Anterior;--guardo l'anterior per despres esborrar-lo
      Seguent(L);
      Allibera_Espai(Antaux);
    end loop;
    Allibera_Espai(L.Anterior);
  end alliberar_memoria;

end llista;

