with Ada.Text_Io; use Ada.Text_Io;
with Taulers; use Taulers;
with Posicions; use Posicions;
with TableroVisual; use TableroVisual;
with Gtk.Main; use Gtk.Main;
with Glib; use Glib;
with Ada.Real_Time; use Ada.Real_Time;
with Sorpreses; use Sorpreses;
with ComecocosPack; use ComecocosPack;
with Partida; use Partida;
with Cleaner; use Cleaner;

procedure start is


  package i_io is new Ada.Text_IO.Integer_IO(integer);  use i_io;
  package f_io is new Ada.Text_IO.Float_IO(float);  use f_io;
  
  package Gint_Timeout is new Gtk.Main.Timeout (PTauler);

  NecesarioArchivo: Exception;
  
  -- Actualiza area de dibujo
  function ActualizarDibujoTimer (T: PTauler) return Boolean is
  begin
    -- L'actualitzaci la fa el Tauler, ningu accedeix al TableroVisual excepte Tauler.
    ActualizarDibujo(T.ALL);
    return true;
  end ActualizarDibujoTimer;
  
  procedure abrirArchivo(nom: in String; file: in out file_type) is
    
  begin
    if nom = "" then
      raise NecesarioArchivo;
    end if;
    open(file, in_file, nom);
  end;
  
  procedure cerrarArchivo(file: in out file_type) is
  begin
    close(file);
  end;
  
  -- Lee la configuracin inicial necesaria para crear un Objeto protegido Tauler.
  -- No se creara el tauler dentro de esta aplicacion. Una vez creado se debe llamar a
  -- "configuracion" para que siga leyendo la configuracin del archivo y la actualice
  -- sobre el tauler creado.
  -- file debe ser un descriptor de archivo ya abierto. No sera cerrado dentro de la funcin.
  procedure conf_inicial(file: file_type; X,Y: out integer; Obstacle, Coco, Vitamina, Ou: out character) is
    c: character;
    --i: Integer;
    bucle: integer;
    error: exception;
  begin
    if (end_of_file(file)) then
      raise error;
    end if;
    
    -- Leemos Simbolo Obstaculo
    bucle := 0;
    while not (end_of_line(file)) and bucle<16 loop
      get(file, c);
      bucle := bucle+1;
    end loop;
    get(file, Obstacle);
    skip_line(file);
    
    -- Leemos Simbolo Coco
    bucle := 0;
    while not (end_of_line(file)) and bucle<12 loop
      get(file, c);
      bucle := bucle+1;
    end loop;
    get(file, Coco);
    skip_line(file);
    
    -- Leemos  Simbolo Vitamina
    bucle := 0;
    while not (end_of_line(file)) and bucle<16 loop
      get(file, c);
      bucle := bucle+1;
    end loop;
    get(file, Vitamina);
    skip_line(file);
    
    -- Leemos Simbolo Huevo
    bucle := 0;
    while not (end_of_line(file)) and bucle<10 loop
      get(file, c);
      bucle := bucle+1;
    end loop;
    get(file, Ou);
    skip_line(file);
    skip_line(file);
    
    -- Leemos Tamao del tablero
    bucle := 0;
    while not (end_of_line(file)) and bucle<14 loop
      get(file, c);
      bucle := bucle+1;
    end loop;
    get(file, Y);
    get(file, c); -- la coma
    get(file, X);
    skip_line(file);
    
    
    EXCEPTION
      WHEN error => put_line("Arxiu de configuraci incorrecte!");
  end conf_inicial;
  
  
  -- Lee el tablero y lo actualiza en T (Las funciones que hacen uso de T se encargaran
  -- de visualizarlo en el TVisual). Ademas lee algunos datos mas que devolver
  -- por parmetro.
  -- file debe ser un descriptor de archivo ya abierto. No sera cerrado dentro de la funcion.
  procedure configuracion(file: in file_type; T: in out Tauler; numMenjacocos: out Integer;
      Num_Partides: out Integer; Temps_Minim_Entre_Moviments, Temps_Maxim_Partida: out Duration; llistaComecocos: out LComecocos.Llista;
      Coord: in out PCoordinador) is
    bucleX, bucleY, i, limitX, limitY, valorOu, valorVitamina, valorCoco: Integer;
    pos: Posicio;
    c: character;
    temps: float;
    llista_pos: Lposicions.Llista;
    PP: PPosicio;
    CC: PComecocos;
    PTminim: PDuration;
  begin
    -- Leemos el Tablero

    bucleY:=0;
    limitX := Obtenir_Num_Columnes(T);
    limitY := Obtenir_Num_Fileres(T);
    while bucleY < limitY loop
      -- Saltamos la definicion
      i := 0;
      while not (end_of_line(file)) and i<17 loop
        get(file, c);
        i := i+1;
      end loop;

      -- Leemos todas las columnas de la fila y lo aadimos al tablero
      bucleX := 0;
      while not (end_of_line(file)) and bucleX < limitX loop
        get(file, c);
        pos.Col := bucleX+1;
        pos.Fil := bucleY+1;
        Afegir_Elem(T, pos, c);
        bucleX := bucleX+1;
      end loop;
      skip_line(file);
      bucleY := bucleY +1;
      
    end loop;

    skip_line(file);

    -- Leemos el valor del coco
    i := 0;
    while not (end_of_line(file)) and i<11 loop
      get(file, c);
      i := i+1;
    end loop;
    get(file, valorCoco);
    skip_line(file);

    -- Leemos el valor de la vitamina
    i := 0;
    while not (end_of_line(file)) and i<15 loop
      get(file, c);
      i := i+1;
    end loop;
    get(file, valorVitamina);
    skip_line(file);

    -- Leemos el valor del huevo
    i := 0;
    while not (end_of_line(file)) and i<9 loop
      get(file, c);
      i := i+1;
    end loop;
    get(file, valorOu);
    skip_line(file);
    
    setValors(T, valorCoco, valorVitamina, valorOu);
    skip_line(file);
    
    -- Leemos el numero me comecocos
    i := 0;
    while not (end_of_line(file)) and i<15 loop
      get(file, c);
      i := i+1;
    end loop;
    get(file, numMenjacocos);
    skip_line(file);

    -- Leemos la posicin de los comecocos
    LPosicions.Crear(llista_pos); --creem una llista de posicions dels menjacocos
    i := 0;
    while not (end_of_line(file)) and i<numMenjacocos loop
      bucleX:=0;
      while not (end_of_line(file)) and bucleX<17 loop
        get(file, c);
        bucleX := bucleX+1;
      end loop;
      get(file, pos.Fil);
      get(file, c);   -- La coma
      get(file, pos.Col);
      Lposicions.Afegir(llista_pos,pos); --afegit a la llista de posicions les coordenaes del comecocos.
      Afegir_Menjacocos(T, pos, false);  -- false = comecoco no creado por incubacion
      skip_line(file);
      i := i+1;
    end loop;
    skip_line(file);
    
    -- Leemos el numero de partidas
    i := 0;
    while not (end_of_line(file)) and i<13 loop
      get(file, c);
      i := i+1;
    end loop;
    get(file, Num_Partides);
    skip_line(file);
    
    -- Leemos la Duracion minima entre movimientos
    i := 0;
    while not (end_of_line(file)) and i<28 loop
      get(file, c);
      i := i+1;
    end loop;
    get(file, temps);
    Temps_Minim_Entre_Moviments := To_Duration(Milliseconds( Integer( temps*Float(1000)) ));
    
    PTMinim:= new Duration;
    PTMinim.ALL:=Temps_Minim_Entre_Moviments;
    LComecocos.Crear(llistaComecocos); --crea llista de comecocos
    Lposicions.Primer(llista_pos); --posa punter anterior al ghost elem
    while Lposicions.Queden(llista_pos) loop --mentre quedin elements (numelem=numcomecocos)
      PP := new Posicio;
      PP.ALL :=Lposicions.Actual(llista_pos); --posicio inicial del comecocos
      -- PP l'utilitzem per poder passar-li la posici al comecocos (ha de ser per punters)
      CC := new Comecocos(Coord,PP,PTMinim, false); --creem el comecocos
                                              -- false = comecoco no incubado
      
      LComecocos.Afegir(llistaComecocos, CC); --l'afegim a la llista
      Lposicions.Seguent(llista_pos); --pasem a la seguent posicio
    end loop;
    
    skip_line(file);
    -- Leemos la duracin mxima de la partida
    i := 0;
    while not (end_of_line(file)) and i<20 loop
      get(file, c);
      i := i+1;
    end loop;
    get(file, temps);
    Temps_Maxim_Partida := To_Duration(Milliseconds( Integer(temps*Float(1000)) ));
    skip_line(file);
  end configuracion;
  
  
TVisual: PTableroVisualRecord;
X, Y, numMenjacocos, Num_Partides: integer;
Obstacle, Coco, Vitamina, Ou: character;
archivo: file_type;
T: PTauler;
Temps_Minim_Entre_Moviments, Temps_Maxim_Partida: PDuration;
GeneradorSorpreses: Psor;
Coord: PCoordinador;
Controller: PControl;
llistaComecocos: PLlista;
RelojPartida: PPartidaTimer;
Id: Timeout_Handler_Id;
GarbageCleaner: PClean;

begin
  Gtk.Main.Set_Locale;
  Gtk.Main.Init;
  -- Config es una funcion del paquete tableroVisual que pedira un
  -- archivo de conf graficamente y devolvera su ruta en un String.
  abrirArchivo(Config, archivo);

  -- Leer configuracion minima (Tamaos de tablero)
  conf_inicial(archivo, X, Y, Obstacle, Coco, Vitamina, Ou);

  llistaComecocos:= new LComecocos.Llista;
  
  -- Creamos el tablero visual
  TVisual := new TableroVisualRecord;
  -- Inicializamos el entorno visual y dibujamos el tablero.
  Init(TVisual.ALL, GInt(X), Gint(Y));
  
  -- Creamos el objeto protegido Tauler a partir de la configuracin mnima obtenida
  T := new Tauler(X, Y, Obstacle, Coco, Vitamina, Ou, TVisual);
  
  
  Draw_Tablero(TVisual.ALL);

  -- Leer resto de configuracion: Ubicacion de los objetos sobre el tablero
  -- y Visualizarla. Y obtenemos algunos parmetros adcionales que tambin se
  -- encuentran en el archivo de configuracin.
  Temps_Minim_Entre_Moviments := new Duration;
  Temps_Maxim_Partida := new Duration;

  Coord := new Coordinador(T);
  GeneradorSorpreses := new Sor(T);
  GarbageCleaner := new Clean(T);
  Controller := new Control(T, Coord, llistaComecocos, GeneradorSorpreses, GarbageCleaner);
  Coord.SetControl(Controller); -- Antes de poder usarlo hay que pasarle el controller
                                -- para que pueda enviarle mensajes de creacin de comecocos
  
  --Crearllista
  configuracion(archivo, T.ALL, numMenjacocos, Num_Partides, Temps_Minim_Entre_Moviments.ALL, Temps_Maxim_Partida.ALL,
  llistaComecocos.ALL, Coord);
  cerrarArchivo(archivo);
  
  -- El RelojPartida inicia el sistema (coordinador, sorpresas y comecocos) a traves de Control
  RelojPartida := new PartidaTimer(Controller, Coord, Temps_Maxim_Partida, Num_Partides, GeneradorSorpreses);
  
  -- Cada 50 ms que se actualice el area de dibujo
  Id := Gint_Timeout.Add (50, ActualizarDibujoTimer'Access, T);
  Gtk.Main.Main;

  EXCEPTION
    WHEN NecesarioArchivo => put_line("Es necessari que indiqui un arxiu de configuraci!");
    WHEN DATA_ERROR => put("Per comenar s'ha d'escollir un fitxer de configuracio correcte!");
    WHEN END_ERROR => put("ERROR: Podria ser que el fitxer de configuracio fos incorrecte!");
    WHEN STORAGE_ERROR => put("ERROR: No queda ms espai a memoria!");
    
end start;
