with Glib;              use Glib;
with Double_Buffer;     use Double_Buffer;
with Gtk.Window;        use Gtk.Window;
with Gtk.Label;         use Gtk.Label;
-----
with Gdk.Font;         use Gdk.Font;
with Gdk.Color;         use Gdk.Color;
with Gdk.Drawable;      use Gdk.Drawable;
with Gdk.Gc;            use Gdk.Gc;
with Gtk.Enums;         use Gtk.Enums;
with Gtk.Handlers;      use Gtk.Handlers;
pragma Elaborate_All (Gtk.Handlers);
with Gtk.Widget;        use Gtk.Widget;
with Gtk.Main;          use Gtk.Main;
with Gtk.Fixed;         use Gtk.Fixed;
pragma Elaborate_All (Gtk.Main);
with Gtk.Button;        use Gtk.Button;
with Gtkada.File_Selection; use Gtkada.File_Selection;
with Ada.Calendar; use Ada.Calendar;
with Parametros; use Parametros;


package body TableroVisual is
  White_Gc : Gdk.Gc.Gdk_GC;
  Black_Gc : Gdk.Gc.Gdk_GC;
  DarkRed_Gc : Gdk.Gc.Gdk_GC;
  Red_Gc   : Gdk.Gc.Gdk_GC;
  Yellow_Gc: Gdk.Gc.Gdk_GC;
  Green_Gc : Gdk.Gc.Gdk_GC;
  Blue_Gc  : Gdk.Gc.Gdk_GC;
  DarkBlue_Gc  : Gdk.Gc.Gdk_GC;
  Gray_Gc  : Gdk.Gc.Gdk_GC;
  Font: Gdk_Font;
  
  Ancho: Constant Gint:=550;  -- Anchura de la ventana de estado
  
  package Handlers is new Gtk.Handlers.Callback (Gtk_Window_Record);
  package Button_Handlers is new Gtk.Handlers.User_Callback (Gtk_Button_Record, Gtk_Window);


  ----------------------------------------------------------------------
  -- FUNCIONES DE DIBUJO --
  ----------------------------------------------------------------------
  
  -- Dibuja un tablero con el tamao indicado, haciendo un clearscreen antes de
  -- nada.
  procedure Draw_Tablero (T: in TableroVisualRecord) is
    Pixmap : Gdk_Drawable;
  begin
    Pixmap:=Get_Pixmap (Gtk_Double_Buffer (T.Buffer));

    -- ClearScreen
    Draw_Rectangle(Pixmap, Gray_Gc, True, 0, 0, (T.Anchura*Parametros.Size), (T.Altura*Parametros.Size));
    -- Recuadro
    Draw_Rectangle(Pixmap, Black_Gc, False, 0, 0, (T.Anchura*Parametros.Size)-1, (T.Altura*Parametros.Size)-1);
      -- Object, Color, X1, Y1, X2, Y2
    for i in 1..T.Anchura-1 loop
      Draw_Line (Pixmap, Black_Gc, Gint(i*Parametros.Size),0,Gint(i*Parametros.Size),T.Altura*Parametros.Size);
    end loop;
    for i in 1..T.Altura-1 loop
      Draw_Line (Pixmap, Black_Gc, 0,Gint(i*Parametros.Size),T.Anchura*Parametros.Size,Gint(i*Parametros.Size));
    end loop;
  end Draw_Tablero;


  -- Dibuja un comecocos en la posicin indicada
  procedure Draw_ComeCoco (T: in TableroVisualRecord; X, Y: Gint; Ident: in integer) is
    Pixmap : Gdk_Drawable;
    id: String := Integer'image(Ident);
  begin
    Pixmap:=Get_Pixmap (Gtk_Double_Buffer (T.Buffer));
    Draw_Casilla(T, X, Y);
      -- X-1 y Y-1 pq la coordenada inicial debe ser (1,1) segun enunciado
    Draw_Arc(Pixmap, DarkRed_Gc, True, (X-1)*Parametros.Size, (Y-1)*Parametros.Size, Parametros.Size-2,Parametros.Size-2, 10*64, 340*64);
    Draw_Text(Pixmap, Font, White_Gc, (X-1)*Parametros.Size+(Parametros.Size/8), (Y-1)*Parametros.Size +(Parametros.Size/2)+(Parametros.Size/3), id);
      -- Object, Color, Filled, X, Y, Width, Height, Angle1, Angle2
  end Draw_ComeCoco;
  
  -- Dibuja un obstaculo en la posicin indicada
  procedure Draw_Obstaculo (T: in TableroVisualRecord; X, Y: Gint) is
    Pixmap : Gdk_Drawable;
  begin
    Pixmap:=Get_Pixmap (Gtk_Double_Buffer (T.Buffer));
    Draw_Casilla(T, X, Y);
    -- X-1 y Y-1 pq la coordenada inicial debe ser (1,1) segun enunciado
    Draw_Rectangle(Pixmap, Black_Gc, True, (X-1)*Parametros.Size, (Y-1)*Parametros.Size, Parametros.Size, Parametros.Size);
    Draw_Rectangle(Pixmap, Gray_Gc, False, ((X-1)*Parametros.Size)+2, ((Y-1)*Parametros.Size)+2, Parametros.Size-4, Parametros.Size-4);
  end Draw_Obstaculo;
  

  -- Dibuja un coco en la posicin indicada
  procedure Draw_Coco (T: in TableroVisualRecord; X, Y: Gint) is
    Pixmap : Gdk_Drawable;
  begin
    Pixmap:=Get_Pixmap (Gtk_Double_Buffer (T.Buffer));
    Draw_Casilla(T, X, Y);
    -- X-1 y Y-1 pq la coordenada inicial debe ser (1,1) segun enunciado
    -- Se realizan todos esos calculos para que quede centrado el coco
    Draw_Arc(Pixmap, Black_Gc, True, ((X-1)*Parametros.Size)+(Parametros.Size/2)-(Parametros.Size/8), ((Y-1)*Parametros.Size)+(Parametros.Size/2)-(Parametros.Size/8), Parametros.Size/4,Parametros.Size/4, 0, 360*64);
      -- Object, Color, Filled, X, Y, Width, Height, Angle1, Angle2
  end Draw_Coco;


  -- Dibuja una vitamina en la posicin indicada
  procedure Draw_Vitamina (T: in TableroVisualRecord; X, Y: Gint) is
    Pixmap : Gdk_Drawable;
  begin
    Pixmap:=Get_Pixmap (Gtk_Double_Buffer (T.Buffer));
    Draw_Casilla(T, X, Y);
    -- X-1 y Y-1 pq la coordenada inicial debe ser (1,1) segun enunciado
    Draw_Rectangle(Pixmap, Green_Gc, True, ((X-1)*Parametros.Size)+(Parametros.Size/2)-(Parametros.Size/8), ((Y-1)*Parametros.Size)+2, Parametros.Size/4, Parametros.Size-4);
    Draw_Rectangle(Pixmap, Green_Gc, True, ((X-1)*Parametros.Size)+2, ((Y-1)*Parametros.Size)+(Parametros.Size/2)-(Parametros.Size/8), Parametros.Size-4, Parametros.Size/4);
  end Draw_Vitamina;
   
  
  -- Dibuja un Huevo en la posicin indicada
  procedure Draw_Huevo (T: in TableroVisualRecord; X, Y: Gint) is
    Pixmap : Gdk_Drawable;
  begin
    Pixmap:=Get_Pixmap (Gtk_Double_Buffer (T.Buffer));
    Draw_Casilla(T, X, Y);
    -- X-1 y Y-1 pq la coordenada inicial debe ser (1,1) segun enunciado
    Draw_Arc(Pixmap, White_Gc, True, ((X-1)*Parametros.Size)+(((Parametros.Size/2)+(Parametros.Size/4))/2)-(Parametros.Size/4), ((Y-1)*Parametros.Size)+2, (Parametros.Size/2)+(Parametros.Size/4),Parametros.Size-4, 0, 360*64);
      -- Object, Color, Filled, X, Y, Width, Height, Angle1, Angle2
  end Draw_Huevo;
  
  
  -- Dibuja una sorpresa en la posicin indicada
  procedure Draw_Sorpresa (T: in TableroVisualRecord; X, Y: Gint) is
    Pixmap : Gdk_Drawable;
  begin
    Pixmap:=Get_Pixmap (Gtk_Double_Buffer (T.Buffer));
    Draw_Casilla(T, X, Y);
    -- X-1 y Y-1 pq la coordenada inicial debe ser (1,1) segun enunciado
    Draw_Arc(Pixmap, DarkBlue_Gc, True, ((X-1)*Parametros.Size)+(Parametros.Size/2)+(Parametros.Size/6), ((Y-1)*Parametros.Size)+(Parametros.Size/2)-(Parametros.Size/4), Parametros.Size/4,Parametros.Size/2, 0, 360*64);
    Draw_Rectangle(Pixmap, DarkBlue_Gc, True, ((X-1)*Parametros.Size)+(Parametros.Size/2)-(Parametros.Size/4), ((Y-1)*Parametros.Size)+(Parametros.Size/2)-(Parametros.Size/4), Parametros.Size/2, Parametros.Size/2);
    Draw_Arc(Pixmap, Blue_Gc, True, ((X-1)*Parametros.Size)+(Parametros.Size/2)-(Parametros.Size/3), ((Y-1)*Parametros.Size)+(Parametros.Size/2)-(Parametros.Size/4), Parametros.Size/4,Parametros.Size/2, 0, 360*64);
      -- Object, Color, Filled, X, Y, Width, Height, Angle1, Angle2
  end Draw_Sorpresa;
  
  
  -- Dibuja una pista en la posicin indicada
  procedure Draw_Pista (T: in TableroVisualRecord; X, Y: Gint) is
    Pixmap : Gdk_Drawable;
    id: String :="?";
  begin
    Pixmap:=Get_Pixmap (Gtk_Double_Buffer (T.Buffer));
    Draw_Casilla(T, X, Y);
    -- X-1 y Y-1 pq la coordenada inicial debe ser (1,1) segun enunciado
    Draw_Rectangle(Pixmap, Yellow_Gc, True, ((X-1)*Parametros.Size)+5, ((Y-1)*Parametros.Size)+5, Parametros.Size-10, Parametros.Size-10);
    Draw_Rectangle(Pixmap, Black_Gc, False, ((X-1)*Parametros.Size)+7, ((Y-1)*Parametros.Size)+7, Parametros.Size-15, Parametros.Size-15);
    Draw_Text(Pixmap, Font, Black_Gc, (X-1)*Parametros.Size+(Parametros.Size/2)-4, (Y-1)*Parametros.Size +(Parametros.Size/5)+(Parametros.Size/2), id);
  end Draw_Pista;

  -- Dibuja una casilla en blanco
  procedure Draw_Casilla (T: in TableroVisualRecord; X, Y: Gint) is
    Pixmap : Gdk_Drawable;
  begin
    Pixmap:=Get_Pixmap (Gtk_Double_Buffer (T.Buffer));
    Draw_Rectangle(Pixmap, Gray_Gc, True, ((X-1)*Parametros.Size)+1, ((Y-1)*Parametros.Size)+1, Parametros.Size-2, Parametros.Size-2);
  end Draw_Casilla;
  
  ----------------------------------------------------------------------
  -- FUNCIONES DE DIBUJO FIN --
  ----------------------------------------------------------------------
  
  
  -- Abre una caja de dialogo para seleccionar un archivo y devuelve la ruta
  -- del archivo seleccionado en un String
  function Config return String is
    File: String:=File_Selection_Dialog("Seleccione el archivo de configuracion", "", False, True);
  begin
    return File;
  end Config;

  
  ----------------------------------------------------------------------
  -- HANDLERS --
  ----------------------------------------------------------------------
  
  -- Pide un archivo de configuracin
  procedure ConfigFile (B: access Gtk_Button_Record'Class; To_Be_Deleted: Gtk_Window) is
    File: String := Config;
  begin
    null;
  end ConfigFile;

  -- Termina
  procedure Quit (Win : access Gtk_Window_Record'Class) is
  --    pragma Warnings (Off, Win);
  begin
      Gtk.Main.Gtk_Exit (0);
  end Quit;

  -- Termina (para llamarlo desde un boton clickado)
  procedure Quit2 (B: access Gtk_Button_Record'Class; To_Be_Deleted: Gtk_Window) is
  begin
    Gtk.Main.Gtk_Exit (0);
  end Quit2;

  -- Debe ser llamado desde un timeout (Gint_Timeout) para que actualice la pantalla.
  -- Si es llamado desde otro lugar, no tendra efecto visible no se pq.
  procedure Actualizar(T:  TableroVisualRecord) is
  begin
    Double_Buffer.Draw(T.Buffer);
  end actualizar;
  ----------------------------------------------------------------------
  -- HANDLERS FIN --
  ----------------------------------------------------------------------

  ----------------------------------------------------------------------  
  ------------------------------- STATUS -------------------------------
  ----------------------------------------------------------------------
  -- Procedure privado
  -- Crea la finestra Status amb un tamany depenent del nombre de menjacocos que
  -- hi ha.
  procedure InitStatus(T: in out TableroVisualRecord) is
    Etiq: Gtk_Label;
  begin
    T.TotalMenjaCocos := 0; 

    Gtk_New (T.StatusWindow, Window_Toplevel);
    Set_Title (T.StatusWindow, "Status");
    Set_Policy (T.StatusWindow, False, False, False); -- Shrink, Grow, Auto
    Set_USize(T.StatusWindow, Ancho, 100+Gint(15*T.TotalMenjaCocos));
    -- Si cerramos ventana, que acabe la aplicacin
    Handlers.Connect (T.StatusWindow, "destroy", Handlers.To_Marshaller (Quit'Access));
    -- Creamos contenedor Fijo, que se pueden aadir todos los widget que queramos
    -- en cualquier posicion
    Gtk_New(T.FixedStatus);
    Add (T.StatusWindow, T.FixedStatus);

    -- Creacio label punts totals
    Gtk_New(T.PuntsTotals, "Punts totals: 0");
    Set_UPosition (T.PuntsTotals, 10, 10);
    Set_Justify(T.PuntsTotals, Justify_Left);
    Put (T.FixedStatus, T.PuntsTotals, 10, 10);
    
    -- Creacio label puntuacio mitja
    Gtk_New(T.PuntuacioMitja, "Puntuacio mitja: 0");
    Set_UPosition (T.PuntuacioMitja, 350, 10);
    Set_Justify(T.PuntuacioMitja, Justify_Left);
    Put (T.FixedStatus, T.PuntuacioMitja, 350, 10);
    
    -- Separador
    Gtk_New(Etiq, "_____________________________________________");
    Set_UPosition (Etiq, 10, 15);
    Set_Justify(Etiq, Justify_Left);
    Put (T.FixedStatus, Etiq, 10, 15);
    Gtk_New(Etiq, "_________________________");
    Set_UPosition (Etiq, 350, 15);
    Set_Justify(Etiq, Justify_Left);
    Put (T.FixedStatus, Etiq, 350, 15);
    
    Show_All (T.StatusWindow);
  end InitStatus;
  
  -- Aade una etiqueta a la ventana de estado
  procedure AfegirMenjacocos(T: in out TableroVisualRecord) is
  begin
    -- Si hay espacio para crear otro label lo hacemos, sino lo ignoramos
    if (T.TotalMenjaCocos < MaxMenjaCocos) then
      
      Gtk_New (T.EstadoComecocos(T.TotalMenjaCocos), "ID " & integer'image(T.TotalMenjaCocos) & " -> Punts: 0 / Caselles: 0 / Temps: 0");
      Set_UPosition (T.EstadoComecocos(T.TotalMenjaCocos), 10, Gint(30+(T.TotalMenjaCocos*10)+(T.TotalMenjaCocos*5)));
      Set_Justify(T.EstadoComecocos(T.TotalMenjaCocos), Justify_Left);
      Put (T.FixedStatus, T.EstadoComecocos(T.TotalMenjaCocos), 10, Gint(30+(T.TotalMenjaCocos*10)+(T.TotalMenjaCocos*5)));
    
      -- El tamao de la ventana ha de ser proporicional al numero de etiquetas
      -- de comecocos o de partidas, de las que haya mas
      if (T.TotalMenjacocos >= T.TotalPartidas-1) then
        Set_USize(T.StatusWindow, Ancho, 100+Gint(15*T.TotalMenjacocos));
      else
        Set_USize(T.StatusWindow, Ancho, 100+Gint(15*T.TotalPartidas-1));
      end if;
      Show(T.EstadoComecocos(T.TotalMenjaCocos));

      T.TotalMenjaCocos := T.TotalMenjaCocos + 1;
    end if;
  end AfegirMenjacocos;
  
  -- Destruye todas las etiquetas de la ventana de estado
  procedure TreureTotsMenjacocos(T: in out TableroVisualRecord) is
  begin
    for i in 0..T.TotalMenjaCocos-1 loop
      Hide(T.EstadoComecocos(i));
      Destroy(T.EstadoComecocos(i));
    end loop;
    T.TotalMenjaCocos := 0;
  end TreureTotsMenjacocos;
  
  -- Cambia el texto de una etiqueta de la ventana de estado
  procedure ActualitzarMenjacocos(T: in out TableroVisualRecord; ID, Punts, Caselles: in integer; Periodo: in Time_Span) is
  begin
    -- Si fa referencia a un menjacocos que no tenim el seu estat visualitzat, ho ignorem
    if (ID < T.TotalMenjaCocos) then
      Set_Text(T.EstadoComecocos(ID), "ID " & integer'image(ID) & " -> Punts: " & integer'image(Punts) &" / Caselles: " & integer'image(Caselles) &" / Temps: " & duration'image(To_Duration(Periodo)));
    end if;
  end ActualitzarMenjacocos;
  
  
  -- Cambia el texto de la etiqueta de puntuacin total de la ventana de estado
  procedure ActualitzarTotal(T: in out TableroVisualRecord; Punts: integer) is
  begin
    Set_Text(T.PuntsTotals, "Punts totals: " & integer'image(Punts));
  end ActualitzarTotal;

  -- Aade una etiqueta de partida a la ventana de estado
  procedure AfegirPartida(T: in out TableroVisualRecord; Punts: integer) is
  begin
    -- Si hay espacio para crear otro label lo hacemos, sino lo ignoramos
    if (T.TotalPartidas < MaxPartides) then
      
      Gtk_New (T.EstadoParidas(T.TotalPartidas), "Partida " & integer'image(T.TotalPartidas) & " -> Punts: " & integer'image(Punts) );
      Set_UPosition (T.EstadoParidas(T.TotalPartidas), 350, Gint(30+(T.TotalPartidas*10)+(T.TotalPartidas*5)));
      Set_Justify(T.EstadoParidas(T.TotalPartidas), Justify_Left);
      Put (T.FixedStatus, T.EstadoParidas(T.TotalPartidas), 350, Gint(30+(T.TotalPartidas*10)+(T.TotalPartidas*5)));
    
      -- El tamao de la ventana ha de ser proporicional al numero de etiquetas
      -- de comecocos o de partidas, de las que haya mas
      if (T.TotalPartidas >= T.TotalMenjacocos-1) then
        Set_USize(T.StatusWindow, Ancho, 100+Gint(15*T.TotalPartidas));
      else
        Set_USize(T.StatusWindow, Ancho, 100+Gint(15*T.TotalMenjacocos-1));
      end if;
      
      Show(T.EstadoParidas(T.TotalPartidas));

      T.TotalPartidas := T.TotalPartidas + 1;
    end if;
  end AfegirPartida;
  
  procedure ActualitzarPuntuacioMitja(T: in out TableroVisualRecord; Punts: float) is
  begin
    Set_Text(T.PuntuacioMitja, "Puntuacio mitja: " & float'image(Punts));
  end ActualitzarPuntuacioMitja;
  ----------------------------------------------------------------------
  ----------------------------- STATUS FIN -----------------------------
  ----------------------------------------------------------------------
  
  
  ----------------------------------------------------------------------
  -- INICIALIZACIN --
  ----------------------------------------------------------------------
  -- Crea ventana, area de dibujo, botones de interaccin con el usuario y
  -- etiquetas de estado.
  -- Al final llama a la inicializacin de la ventana de estado.
  procedure Init(T: in out TableroVisualRecord; Anchura: in Gint; Altura: in Gint) is
    Fixed  : Gtk_Fixed;
    Color: Gdk_Color;
    Button : Gtk_Button;
  begin
    T.Anchura:= Anchura;
    T.Altura := Altura;

    Gtk_New (T.Window, Window_Toplevel);
    Set_Title (T.Window, "GNU Comecocos");
    Set_Policy (T.Window, False, False, False); -- Shrink, Grow, Auto
    Set_USize(T.Window, (T.Anchura*Parametros.Size)+20, (T.Altura*Parametros.Size)+42);
    -- Si cerramos ventana, que acabe la aplicacin
    Handlers.Connect (T.Window, "destroy", Handlers.To_Marshaller (Quit'Access));
    -- Creamos contenedor Fijo, que se pueden aadir todos los widget que queramos
    -- en cualquier posicion
    Gtk_New(Fixed);
    

    -- Nuevo DobleBuffer (el solo se encarga de hacer doble buffering)
    Gtk_New (T.Buffer);
    Set_USize (T.Buffer, (T.Anchura*Parametros.Size), (T.Altura*Parametros.Size));

    Put (Fixed, T.Buffer, 5, 5);
    Add (T.Window, Fixed);

    -- Creacin de todos los botones
    Gtk_New (Button, "Salir");
    Set_Relief (Button, Relief_Normal);
    Set_UPosition (Button, (T.Anchura*Parametros.Size)-47, (T.Altura*Parametros.Size)+10);
    Set_USize (Button, 47, 22);
    Put (Fixed, Button, (T.Anchura*Parametros.Size)-47, (T.Altura*Parametros.Size)+10);

    Button_Handlers.Connect (Button, "clicked", Button_Handlers.To_Marshaller (Quit2'Access),T.Window);

    -- Mostramos todo
    Show_All (T.Window);
    
    -- Cargamos la fuente
    Load (Font, "-adobe-helvetica-bold-r-normal--14-140-75-75-p-78-iso8859-1");
    --  Generacin de los colores
    Color := Gdk.Color.Parse ("Black");
    Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
    Gdk.Gc.Gdk_New (Black_Gc, Get_Window (T.Buffer));
    Gdk.Gc.Set_Foreground (Black_Gc, Color);
    
    Color := Gdk.Color.Parse ("White");
    Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
    Gdk.Gc.Gdk_New (White_Gc, Get_Window (T.Buffer));
    Gdk.Gc.Set_Foreground (White_Gc, Color);
      
    Color := Gdk.Color.Parse ("Red");
    Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
    Gdk.Gc.Gdk_New (Red_Gc, Get_Window (T.Buffer));
    Gdk.Gc.Set_Foreground (Red_Gc, Color);
      
    Color := Gdk.Color.Parse ("#DD0000");
    Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
    Gdk.Gc.Gdk_New (DarkRed_Gc, Get_Window (T.Buffer));
    Gdk.Gc.Set_Foreground (DarkRed_Gc, Color);
      
    Color := Gdk.Color.Parse ("#F7DE0D");
    Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
    Gdk.Gc.Gdk_New (Yellow_Gc, Get_Window (T.Buffer));
    Gdk.Gc.Set_Foreground (Yellow_Gc, Color);
      
    Color := Gdk.Color.Parse ("#00AA00");
    Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
    Gdk.Gc.Gdk_New (Green_Gc, Get_Window (T.Buffer));
    Gdk.Gc.Set_Foreground (Green_Gc, Color);
      
    Color := Gdk.Color.Parse ("#5555FF");
    Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
    Gdk.Gc.Gdk_New (Blue_Gc, Get_Window (T.Buffer));
    Gdk.Gc.Set_Foreground (Blue_Gc, Color);
      
    Color := Gdk.Color.Parse ("#000077");
    Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
    Gdk.Gc.Gdk_New (DarkBlue_Gc, Get_Window (T.Buffer));
    Gdk.Gc.Set_Foreground (DarkBlue_Gc, Color);
      
    Color := Gdk.Color.Parse ("Gray");
    Gdk.Color.Alloc (Gtk.Widget.Get_Default_Colormap, Color);
    Gdk.Gc.Gdk_New (Gray_Gc, Get_Window (T.Buffer));
    Gdk.Gc.Set_Foreground (Gray_Gc, Color);

    -- Inicializamos la ventana de estado
    InitStatus(T);
  end Init;
  ----------------------------------------------------------------------
  -- INICIALIZACIN FIN --
  ----------------------------------------------------------------------
  
  
end TableroVisual;
