-----------------------------
-- Programaci Concurrent  --
--                         --
-- 20-Nov-2002             --
--                         --
-- Sergio Gomez            --
-- Alex Arenas             --
-----------------------------

with TableroVisual; use TableroVisual;
with Posicions; use Posicions;
with Ada.Real_Time; use Ada.Real_Time;

package Taulers is

  ------------
  -- Tauler --
  ------------

  type Tauler(X,Y: positive; Obstacle, Coco, Vitamina, Ou: character; TV: PTableroVisualRecord) is limited private;
  type PTauler is access Tauler;
  
  procedure ActualizarDibujo(T: in out Tauler);
  
  function Obtenir_Num_Fileres(T: in Tauler) return Positive;
  function Obtenir_Num_Columnes(T: in Tauler) return Positive;
  function getValorCoco(T: in Tauler) return Integer;
  function getValorVitamina(T: in Tauler) return Integer;
  function getValorOu(T: in Tauler) return Integer;
  
  procedure Posicions_Obstacles(T: in out Tauler; L: out LPosicions.Llista);
  procedure Posicions_Menjacocos(T: in out Tauler; L: out LPosicions.Llista);
  procedure Posicions_Ous(T: in out Tauler; L: out LPosicions.Llista);
  procedure Posicions_Vitamines(T: in out Tauler; L: out LPosicions.Llista);
  procedure Posicions_Cocos(T: in out Tauler; L: out LPosicions.Llista);  
  function Total_Menjacocos(T: Tauler) return integer;
  
  function Valor_Sorpresa(T: Tauler) return integer;
  function Posicio_Pista(T: Tauler) return Posicio;
  function Posicio_Sorpresa(T: Tauler) return Posicio;
  function Hi_Ha_Sorpresa(T: Tauler) return boolean;
  function Hi_Ha_Pista(T: Tauler) return boolean;
  
  procedure Afegir_OuDolent(T: in out Tauler; P: in Posicio);
  procedure Afegir_VitaminaDolenta(T: in out Tauler; P: in Posicio);
  
  -- Retorna sempre true excepte si ya hem agafat la pista
  -- y sabem que es bona.
  function SorpresaDolenta(T: Tauler) return boolean;
  
  procedure Afegir_Sorpresa(T: in out Tauler; Posicio_Sorpresa: in Posicio;
    Posicio_Pista: in Posicio; Valor_Sorpresa: in Integer);
  procedure Retirar_Sorpresa(T: in out Tauler);

  procedure Afegir_Elem(T:in out Tauler; P: in Posicio; Elem: in character);
  
  procedure setValors(T: in out Tauler; Coco, Vitamina, Ou: in Integer);
  
  procedure Afegir_Menjacocos(T: in out Tauler; P: in Posicio; incubado: in boolean);

  --Incubando devolvera true si el comecoco ha estado encima de un huevo (tb crear una entrada en la ventana Status). 
  --El coordinador crear la nueva taska e iniciarla.
  --Periodo es necesario para poder actualizar el tiempo de movimiento en la ventana de Status.
  procedure Modificar_Menjacocos(T: in out Tauler; AntigaP, P: in Posicio; incubado: out boolean; Periodo: in Time_Span);
  procedure Reset(T: in out Tauler);
  
private

  ------------
  -- Tauler --
  ------------

  type matriz is array (Positive range <>,Positive range <>) of character;

  protected type Tauler (X,Y: positive; Obstacle, Coco, Vitamina, Ou: character; TV: PTableroVisualRecord) is
    procedure ActualizarDibujo;
    
    function Obtenir_Num_Fileres return Positive;
    function Obtenir_Num_Columnes return Positive;
    function getValorCoco return Integer;
    function getValorVitamina return Integer;
    function getValorOu return Integer;

    procedure Posicions_Obstacles(L: out LPosicions.Llista);
    procedure Posicions_Menjacocos(L: out LPosicions.Llista);
    procedure Posicions_Ous(L: out LPosicions.Llista);
    procedure Posicions_Vitamines(L: out LPosicions.Llista);
    procedure Posicions_Cocos(L: out LPosicions.Llista);    
    function Total_Menjacocos return integer;
    
    procedure Afegir_Sorpresa(Posicio_Sorpresa: in Posicio; Posicio_Pista: in Posicio; Valor_Sorpresa: in Integer);
    procedure Retirar_Sorpresa;
    function Valor_Sorpresa return integer;
    function Posicio_Pista return Posicio;
    function Posicio_Sorpresa return Posicio;
    function Hi_Ha_Sorpresa return boolean;
    function Hi_Ha_Pista return boolean;
    
    procedure Afegir_OuDolent(P: in Posicio);
    procedure Afegir_VitaminaDolenta(P: in Posicio);
    
    -- Retorna sempre true excepte si ya hem agafat la pista
    -- y sabem que es bona.
    function SorpresaDolenta return boolean;
  
    procedure Afegir_Elem(P: in Posicio; Elem: in character);
    
    procedure setValors(Coco, Vitamina, Ou: in Integer);
    procedure Afegir_Menjacocos(P: in Posicio; incubado: in boolean);
    procedure Modificar_Menjacocos(AntigaP, P: in Posicio; incubado: out boolean; Periodo: in Time_Span);
    procedure Reset;
    
  private
    TVisual: TableroVisualRecord := TV.ALL;
    -- iniTauler nos servira para resetear todo el panel con sus obstaculos iniciales.
    iniTauler, tauler: matriz(1..X, 1..Y);  -- iniTauler para reiniciar el juego
    -- iniMenjacocos nos servira para resetear los comecocos a sus posiciones iniciales
    -- ousDolents i vitaminesDolentes guardaran les posicions dels ous i vitamines inaccessibles
    iniMenjacocos, ousDolents, vitaminesDolentes: LPosicions.Llista;
    menjacocos: LStatus.Llista;
    totalMenjacocos, totalIniMenjacocos: integer := 0;
    totalPunts, puntsAcumulats, resets: integer := 0;
    sorpresa: boolean := False;  -- Si hi ha sorpresa estara a true.
    pista: boolean := False;  -- Si hi ha pista estar a true (noms valid si sopresa esta a true)
    sorDolenta: boolean := True; -- La sorpresa es dolenta fins que no es demostri lo contrari
    posSorpresa: posicio;  -- La sorpresa i pista no es guarden al tauler
    valorCoco, valorVitamina, valorOu: Integer := 0; 
    posPista: posicio;
    valorSorpresa: integer;
    symObstacle: character := Obstacle;
    symCoco: character := Coco;
    symVitamina: character := Vitamina;
    symOu: character := Ou;
  end Tauler;


end Taulers;
