-----------------------------
-- Programaci Concurrent  --
--                         --
-- 15-Gen-2003             --
--                         --
-- Sergio Gomez            --
-- Alex Arenas             --
-----------------------------

with Ada.Real_Time; use Ada.Real_Time;
with Ada.Numerics.Float_Random; use Ada.Numerics.Float_Random;
with Posicions; use Posicions;
with Taulers; use Taulers;

package body Sorpreses.Generador is

  Seed: Generator;
  Crida_Num: Natural := 0;

  function Posicio_Aleatoria(Num_Fil, Num_Col: in Positive) return Posicio is
    P: Posicio;
  begin
    P.Fil := 1 + (Integer(Float(Num_Fil) * Random(Seed)) mod Num_Fil);
    P.Col := 1 + (Integer(Float(Num_Col) * Random(Seed)) mod Num_Col);
    return P;
  end Posicio_Aleatoria;

  procedure Generar_Sorpresa(T: in out Tauler;
      Posicio_Sorpresa: out Posicio;
      Posicio_Pista: out Posicio;
      Valor_Sorpresa: out Integer;
      Instant_Inici_Sorpresa: out Time;
      Instant_Fi_Sorpresa: out Time) is
    Num_Fil, Num_Col: Positive;
    Obstacles, Menjacocos: LPosicions.Llista;
    P: Posicio;
    Versio: Natural range 0..3;
  begin
    Num_Fil := Obtenir_Num_Fileres(T);
    Num_Col := Obtenir_Num_Columnes(T);
    Posicions_Obstacles(T, Obstacles);
    Posicions_Menjacocos(T, Menjacocos);
    loop
      P := Posicio_Aleatoria(Num_Fil, Num_Col);
      exit when (not LPosicions.Existeix(Obstacles, P)) and
                (not LPosicions.Existeix(Menjacocos, P));
    end loop;
    Posicio_Sorpresa := P;
    loop
      P := Posicio_Aleatoria(Num_Fil, Num_Col);
      exit when (P /= Posicio_Sorpresa) and
                (not LPosicions.Existeix(Obstacles, P)) and
                (not LPosicions.Existeix(Menjacocos, P));
    end loop;
    Posicio_Pista:= P;
    Versio := Crida_Num mod 4;
    case Versio is
      when 0 =>
        Instant_Inici_Sorpresa := Clock + To_Time_Span(1.0);
        Instant_Fi_Sorpresa := Instant_Inici_Sorpresa + To_Time_Span(2.0);
        Valor_Sorpresa := 30;
      when 1 =>
        Instant_Inici_Sorpresa := Clock + To_Time_Span(2.0);
        Instant_Fi_Sorpresa := Instant_Inici_Sorpresa + To_Time_Span(10.0);
        Valor_Sorpresa := 10;
      when 2 =>
        Instant_Inici_Sorpresa := Clock + To_Time_Span(3.0);
        Instant_Fi_Sorpresa := Instant_Inici_Sorpresa + To_Time_Span(5.0);
        Valor_Sorpresa := 20;
      when 3 =>
        Instant_Inici_Sorpresa := Clock + To_Time_Span(7.0);
        Instant_Fi_Sorpresa := Instant_Inici_Sorpresa + To_Time_Span(10.0);
        Valor_Sorpresa := -20;
    end case;
    Crida_Num := Crida_Num + 1;
    LPosicions.Alliberar_Memoria(Obstacles);
    LPosicions.Alliberar_Memoria(Menjacocos);
	
  end Generar_Sorpresa;

begin
  Reset(Seed, 123);
end Sorpreses.Generador;
